<?
	include_once "IPSLogger.ips.php";
	include_once "IPSShadowing_Configuration.ips.php";
	include_once "IPSShadowing_Control.ips.php";
	include_once "IPSShadowing_Hardware.ips.php";
	include_once "IPSShadowing_Custom.ips.php";


	// ----------------------------------------------------------------------------------------------------------------------------
   function IPSShadowing_MoveByCommand($Command, $DeviceId) {

		switch ($Command) {
			case c_MovementId_Up;
			case c_MovementId_Down:
			case c_MovementId_Stop:
		   	IPSShadowing_MoveByHardware($DeviceId, $Command);
		   	break;
			default: 
		}

		if (GetValue(get_ControlId(c_Control_Movement, $DeviceId)) <> $Command) {
		   SetValue(get_ControlId(c_Control_Movement, $DeviceId), $Command);
		}
   }

	// ----------------------------------------------------------------------------------------------------------------------------
   function IPSShadowing_MoveByEvent($DeviceId, $Level) {
		IPSLogger_Dbg(__file__, "Received StateChange from Shutter '".IPS_GetName($DeviceId)."', NewLevel=".round($Level).", CurrentLevel=".GetValue(get_ControlId(c_Control_Position, $DeviceId)));
      if (GetValue(get_ControlId(c_Control_Position, $DeviceId)) <> $Level and
		    GetValue(get_ControlId(c_Control_StepsToDo, $DeviceId))=="") {
			IPSLogger_Inf(__file__, "Apply StateChange from Shutter '".IPS_GetName($DeviceId)."', Level=".round($Level));
		   SetValue(get_ControlId(c_Control_Movement, $DeviceId), c_MovementId_Stop);
		   SetValue(get_ControlId(c_Control_Position, $DeviceId), $Level);
	      if (!GetValue(get_ControlId(c_Control_ManualChange, $DeviceId)) and
			    GetValue(get_ControlId(c_Control_Automatic, $DeviceId))) {
	      	SetValue(get_ControlId(c_Control_ManualChange, $DeviceId), true);
	      }
			IPSShadowing_SetStatus($DeviceId);
	   }
   }

	// ----------------------------------------------------------------------------------------------------------------------------
   function IPSShadowing_MoveByProgram($DeviceId, $ProgramId, $DimoutOption=null, $TriggeredByTemp=false, $isDay=null) {
		$MovementStatus = GetValue(get_ControlId(c_Control_Movement, $DeviceId));

		$DoBeMoved = $MovementStatus;
      switch ($ProgramId) {
			case c_ProgramId_Manual:
			   break;
			case c_ProgramId_Opened:
			   $DoBeMoved = c_MovementId_Opened;
			   break;
			case c_ProgramId_OpenedOrShadowing:
			   if ($MovementStatus<>c_MovementId_Opened and $MovementStatus<>c_MovementId_Shadowing) {$DoBeMoved = c_MovementId_Shadowing;}
			   break;
			case c_ProgramId_OpenedDay:
			   if ($isDay) { $DoBeMoved = c_MovementId_Opened;}
			   break;
			case c_ProgramId_OpenedNight:
			   if (!$isDay) { $DoBeMoved = c_MovementId_Opened;}
			   break;
			case c_ProgramId_Closed:
			   $DoBeMoved = c_MovementId_Closed;
			   break;
			case c_ProgramId_Dimout:
			   $DoBeMoved = c_MovementId_Dimout;
			   break;
			case c_ProgramId_DimoutOrShadowing:
			   if ($MovementStatus<>c_MovementId_Dimout and $MovementStatus<>c_MovementId_Shadowing) {$DoBeMoved = c_MovementId_Shadowing;}
			   break;
			case c_ProgramId_DimoutAndShadowing:
			   if ($DimoutOption) {
				   $DoBeMoved = c_MovementId_Dimout;
			   } else {
				   $DoBeMoved = c_MovementId_Shadowing;
			   }
			   break;
			default:
				IPSLogger_Err(__file__, "Unknown Program $Program, DeviceId=$DeviceId");
				exit;
      }
		if ($DoBeMoved<>$MovementStatus) {
			if ($TriggeredByTemp and !GetValue(get_ControlId(c_Control_TempChange, $DeviceId))) {
				SetValue(get_ControlId(c_Control_TempChange, $DeviceId), true);
			}
			SetValue(get_ControlId(c_Control_Movement, $DeviceId), $DoBeMoved);
			IPSShadowing_Move($DeviceId);
			IPSShadowing_LogMoveByProgram($DeviceId, $ProgramId, $TriggeredByTemp);
		}
	}

	// ----------------------------------------------------------------------------------------------------------------------------
   function IPSShadowing_MoveByControl($ControlId, $Value) {
   	$ControlType = get_ControlType($ControlId);
   	$DeviceId    = get_DeviceIdByControlId($ControlId);
   	
		switch ($ControlType) {
			case c_Control_Movement:
		   	if ($Value==c_MovementId_Space) {
					return;
				} elseif (GetValue($ControlId)==$Value) {
					return;
				} else {
			      if (!GetValue(get_ControlId(c_Control_ManualChange, $DeviceId))) {
			      	SetValue(get_ControlId(c_Control_ManualChange, $DeviceId), true);
			      }
			      SetValue($ControlId, $Value);
				   IPSShadowing_Move($DeviceId);
					IPSShadowing_LogMoveByControl($DeviceId);
				}

			   break;
			default:
				IPSLogger_Err(__file__, "Unknown ControlType $ControlType, DeviceId=$DeviceId");
				exit;
		}
   }

	// ----------------------------------------------------------------------------------------------------------------------------
   function IPSShadowing_Move($DeviceId) {
      IPSShadowing_CalcNextSteps($DeviceId);
     	SetValue(get_ControlId(c_Control_Movement, $DeviceId), -1);
      IPSShadowing_ExecuteNextStep($DeviceId);
      IPSShadowing_StartRefreshTimer(true);
   }

	// ----------------------------------------------------------------------------------------------------------------------------
   function IPSShadowing_AddNextStep($StepsToDo, $Command, $SecondsToDo, $Display, $SecondsTotal, $PercentagePosition) {
      $Step = count($StepsToDo);
      $StepsToDo[$Step]   = $Command;
      $StepsToDo[$Step+1] = $SecondsToDo;
      $StepsToDo[$Step+2] = $Display;
      $StepsToDo[$Step+3] = $SecondsTotal;
      $StepsToDo[$Step+4] = $PercentagePosition;
	}
	
	// ----------------------------------------------------------------------------------------------------------------------------
   function IPSShadowing_CalcNextSteps($DeviceId) {
      $DeviceName   = get_DeviceNameByDeviceId($DeviceId);
      $DeviceConfig = get_ShadowingConfiguration();

      $Position     = GetValue(get_ControlId(c_Control_Position, $DeviceId));
      $ToBeMoved    = GetValue(get_ControlId(c_Control_Movement, $DeviceId));
      $StepsToDo    = array();

		//IPSShadowing_AddNextStep(&$StepsToDo, $ToBeMoved, 1,  '', null, null);
		if ($ToBeMoved==c_MovementId_Opened or $ToBeMoved==c_MovementId_Up) {
			$SecTotal     = $DeviceConfig[$DeviceName][c_Property_TimeOpening];
			$SecNullToPos = $SecTotal*$Position/100;
			$SecPosTo100  = $SecTotal-$SecNullToPos;
			IPSShadowing_AddNextStep(&$StepsToDo, c_MovementId_Up, $SecTotal-$SecPosTo100, null,    $SecTotal, $SecPosTo100);
			IPSShadowing_AddNextStep(&$StepsToDo, c_MovementId_Stop,   $DeviceConfig[$DeviceName][c_Property_TimePause],  'Offen (Stop)', null, null);
			IPSShadowing_AddNextStep(&$StepsToDo, c_MovementId_Opened, 1, null, null , null);

		} elseif ($ToBeMoved==c_MovementId_Shadowing or $ToBeMoved==c_MovementId_Down or $ToBeMoved==c_MovementId_Dimout or $ToBeMoved==c_MovementId_Closed) {
			$SecTotal     = $DeviceConfig[$DeviceName][c_Property_TimeClosing];
			$SecNullToPos = $SecTotal*$Position/100;
	      if ($ToBeMoved==c_MovementId_Dimout) {
				IPSShadowing_AddNextStep(&$StepsToDo, c_MovementId_Down,   $SecTotal-$SecNullToPos, null, $SecTotal, $SecNullToPos);
				IPSShadowing_AddNextStep(&$StepsToDo, c_MovementId_Stop,   $DeviceConfig[$DeviceName][c_Property_TimePause],  'Abdunkelung (Pause)', null, null);
				IPSShadowing_AddNextStep(&$StepsToDo, c_MovementId_Up,     $DeviceConfig[$DeviceName][c_Property_TimeDimoutUp], 'Abdunkelung (Hoch)', null, null);
				IPSShadowing_AddNextStep(&$StepsToDo, c_MovementId_Stop,   $DeviceConfig[$DeviceName][c_Property_TimePause],  'Abdunkelung (Pause)', null, null);
				IPSShadowing_AddNextStep(&$StepsToDo, c_MovementId_Down,   $DeviceConfig[$DeviceName][c_Property_TimeDimoutDown], 'Abdunkelung (Runter)', null, null);
				IPSShadowing_AddNextStep(&$StepsToDo, c_MovementId_Stop,   $DeviceConfig[$DeviceName][c_Property_TimePause],  'Abdunkelung (Stop)', null, null);
				IPSShadowing_AddNextStep(&$StepsToDo, c_MovementId_Dimout, 1,  null, null, null);
	      } elseif ($ToBeMoved==c_MovementId_Down) {
				IPSShadowing_AddNextStep(&$StepsToDo, c_MovementId_Down, $SecTotal-$SecNullToPos, null, $SecTotal, $SecNullToPos);
				IPSShadowing_AddNextStep(&$StepsToDo, c_MovementId_Stop, 1, null, null, null);
			} elseif ($ToBeMoved==c_MovementId_Shadowing) {
				IPSShadowing_AddNextStep(&$StepsToDo, c_MovementId_Down, $SecTotal-$SecNullToPos, null, $SecTotal, $SecNullToPos);
				IPSShadowing_AddNextStep(&$StepsToDo, c_MovementId_Stop, 1, "$Position%", null, null);
				IPSShadowing_AddNextStep(&$StepsToDo, c_MovementId_Up,    $DeviceConfig[$DeviceName][c_Property_TimeDimoutUp], 'Beschattung (Hoch)', null, null);
				IPSShadowing_AddNextStep(&$StepsToDo, c_MovementId_Stop, $DeviceConfig[$DeviceName][c_Property_TimePause],  'Beschattung (Stop)', null, null);
				IPSShadowing_AddNextStep(&$StepsToDo, c_MovementId_Shadowing, 1,  null, null, null);
			} else {
				IPSShadowing_AddNextStep(&$StepsToDo, c_MovementId_Down, $SecTotal-$SecNullToPos, null, $SecTotal, $SecNullToPos);
				IPSShadowing_AddNextStep(&$StepsToDo, c_MovementId_Stop, 1, "$Position%", null, null);
				IPSShadowing_AddNextStep(&$StepsToDo, c_MovementId_Closed, 1,  null, null, null);
			}
			
		} elseif ($ToBeMoved==c_MovementId_75 or $ToBeMoved==c_MovementId_50 or $ToBeMoved==c_MovementId_25) {
			$SecTotal     = $DeviceConfig[$DeviceName][c_Property_TimeClosing];
			$SecNullToPos = $SecTotal*$Position/100;
			$SecPosTo100  = $SecTotal-$SecNullToPos;
			if ($ToBeMoved==c_MovementId_75) {
				$SecNullToNew = $SecTotal*75/100;
				$Position     = 75;
			} elseif ($ToBeMoved==c_MovementId_50) {
				$SecNullToNew = $SecTotal*50/100;
				$Position     = 50;
			} else {
				$SecNullToNew = $SecTotal*25/100;
				$Position     = 25;
			}
			if ($SecNullToNew > $SecNullToPos) {
				IPSShadowing_AddNextStep(&$StepsToDo, c_MovementId_Down, $SecNullToNew-$SecNullToPos, null, $SecTotal, $SecNullToPos);
			} elseif ($SecNullToNew < $SecNullToPos) {
				IPSShadowing_AddNextStep(&$StepsToDo, c_MovementId_Up, $SecNullToPos-$SecNullToNew, null, $SecTotal, $SecPosTo100);
			} else {
			}
			IPSShadowing_AddNextStep(&$StepsToDo, c_MovementId_Stop, 1, "$Position%", null, null);
			IPSShadowing_AddNextStep(&$StepsToDo, $ToBeMoved,        1, null, null, null);

		} elseif ($ToBeMoved==c_MovementId_Stop) {
			IPSShadowing_AddNextStep(&$StepsToDo, c_MovementId_Stop, 1, null, null, null);

		} else {
			IPSLogger_Err(__file__, "Unknown MovementId $ToBeMoved, DeviceId=$DeviceId");
			exit;
		}

      SetValue(get_ControlId(c_Control_StepsToDo, $DeviceId), implode('|', $StepsToDo));
      SetValue(get_ControlId(c_Control_Step,      $DeviceId), -5);
   }

	// ----------------------------------------------------------------------------------------------------------------------------
   function IPSShadowing_ExecuteNextStep($DeviceId) {
      $DeviceName    = get_DeviceNameByDeviceId($DeviceId);
		$Step          = GetValue(get_ControlId(c_Control_Step, $DeviceId))+5;

      $NextStepsToDo = GetValue(get_ControlId(c_Control_StepsToDo, $DeviceId));
      $NextStepsToDo = Explode('|', $NextStepsToDo);

		if ($Step < count($NextStepsToDo)) {
			$Command = $NextStepsToDo[$Step];
			$Time    = $NextStepsToDo[$Step+1];
			IPSLogger_Trc(__file__, "Shadowing for Device '$DeviceName', Step $Step, Command=$Command, Time=$Time");
			IPSShadowing_MoveByCommand($Command, $DeviceId);

	      SetValue(get_ControlId(c_Control_Step, $DeviceId),      $Step);
	      SetValue(get_ControlId(c_Control_StartTime, $DeviceId), time());
		} else {
			IPSLogger_Dbg(__file__, "Finished all Steps for Device '$DeviceName'");
	      SetValue(get_ControlId(c_Control_StepsToDo, $DeviceId), "");
	      SetValue(get_ControlId(c_Control_Step, $DeviceId),      -1);
	      SetValue(get_ControlId(c_Control_StartTime, $DeviceId), -1);
	      IPSShadowing_StartRefreshTimer(false);
	      IPSShadowing_SetStatus($DeviceId);
		}
   }

	// ----------------------------------------------------------------------------------------------------------------------------
	function IPSShadowing_Refresh() {
	   $DeviceIds = IPS_GetChildrenIds(c_ID_Devices);
		foreach($DeviceIds as $DeviceId) {
		   if (GetValue(get_ControlId(c_Control_Step, $DeviceId)) >= 0) {
      		$NextStepsToDo = Explode('|', GetValue(get_ControlId(c_Control_StepsToDo, $DeviceId)));
      		$StepCount     = count($NextStepsToDo);
      		$Step          = GetValue(get_ControlId(c_Control_Step, $DeviceId));
      		$StartTime     = GetValue(get_ControlId(c_Control_StartTime, $DeviceId));
				$SecsDone      = time()-$StartTime;
				$SecsToDo      = $NextStepsToDo[$Step+1];
				$Display       = $NextStepsToDo[$Step+2];
				$SecsTotal     = $NextStepsToDo[$Step+3];
				$SecStepBegin  = $NextStepsToDo[$Step+4];
				$Command       = $NextStepsToDo[$Step];

				if ($SecsTotal <> null) {
					//  SecTotal   ... 100%
					//  Begin+Done ...   x%
   	         $Position       = round(($SecStepBegin+$SecsDone)*100/$SecsTotal);
   	         if ($Command==c_MovementId_Up) {
   	            $Position = 100-$Position;
   	         }
   	         if ($Position>100) {$Position=100;}
   	         if ($Position<0)   {$Position=0;}
	      		SetValue(get_ControlId(c_Control_Position, $DeviceId), $Position);
					$SecsOpen      = $SecsToDo-$SecsDone;
					if ($SecsOpen < 0) {$SecsOpen=0;}
					$Display = "$Position% ($SecsOpen Sek)";
				}

				if ($Display!=null) {
	      		SetValue(get_ControlId(c_Control_Display, $DeviceId), $Display);
				}

				if ($SecsDone >= $SecsToDo) {
			      IPSShadowing_ExecuteNextStep($DeviceId);
				}
		   }
		}
	}

	// ----------------------------------------------------------------------------------------------------------------------------
   function IPSShadowing_StartRefreshTimer($Value) {
	   $Name    = 'Refresh';
	   $TimerId = @IPS_GetEventIDByName($Name, c_ID_RefreshTimerScript);
	   if ($TimerId === false) {
 			$TimerId = IPS_CreateEvent(1 /*Cyclic Event*/);
   		IPS_SetName($TimerId, $Name);
   		IPS_SetParent($TimerId, c_ID_RefreshTimerScript);
			if (!IPS_SetEventCyclic($TimerId, 2 /*Daily*/, 1 /*Int*/,0 /*Days*/,0/*DayInt*/,1/*TimeType Sec*/,1/*Sec*/)) {
				IPSLogger_Err(__file__, "IPS_SetEventCyclic failed for Refresh Timer!!!");
				exit;
			}
		}
		if ($Value) {
	   	IPS_SetEventActive($TimerId, true);
		} else {
		   $OneOrMoreDevicesActive = false;
		   $DeviceIds             = IPS_GetChildrenIds(c_ID_Devices);
			foreach($DeviceIds as $DeviceId) {
			   $OneOrMoreDevicesActive = ($OneOrMoreDevicesActive or GetValue(get_ControlId(c_Control_StepsToDo, $DeviceId))<>null);
			}
			if (!$OneOrMoreDevicesActive) {
		   	IPS_SetEventActive($TimerId, false);
			}
   	}
   }

?>